'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _babelCore = require('babel-core');

var babel = _interopRequireWildcard(_babelCore);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var t = babel.types;
exports.default = getLiteralizers;


function getLiteralizers(_ref) {
  var file = _ref.file;

  return toLiteral;

  function objectToLiteral(path) {
    var props = path.get('properties').map(function (propPath) {
      var propNodeClone = t.clone(propPath.node);
      var valPath = propPath.get('value');
      propNodeClone.shorthand = false;
      propNodeClone.value = toLiteral(valPath);
      return propNodeClone;
    });
    return t.objectExpression(props);
  }

  function arrayToLiteral(path) {
    var els = path.get('elements').map(toLiteral);
    return t.arrayExpression(els);
  }

  function identifierToLiteral(path) {
    var binding = path.scope.getBinding(path.node.name);
    if (binding.path.type.startsWith('Import')) {
      return importToLiteral(binding.path);
    }
    var initPath = binding.path.get('init');
    return toLiteral(initPath);
  }

  function importToLiteral(path) {
    var importedFile = _path2.default.resolve(_path2.default.dirname(file.opts.filename),
    // TODO: support any extension and import style
    `${path.parent.source.value}.js`);
    var literalValue = void 0;
    babel.transformFileSync(importedFile, {
      plugins: [function () {
        var thingToLiteralize = void 0;
        return {
          visitor: {
            // TODO: support other exports
            ExportNamedDeclaration(namedDeclarationPath) {
              thingToLiteralize = namedDeclarationPath.get('declaration.declarations')[0].get('init');
            },
            Program: {
              exit() {
                literalValue = toLiteral(thingToLiteralize);
              }
            }
          }
        };
      }]
    });
    return literalValue;
  }

  function memberExpressionToLiteral(path) {
    var pathProperty = path.get('property');
    var literalProperty = void 0;
    var pathPropertyValue = getPathPropertyValue();
    var literalObject = toLiteral(path.get('object'));
    if (t.isObjectExpression(literalObject)) {
      var literalObjectProperty = literalObject.properties.find(function (prop) {
        return prop.key.name === pathPropertyValue;
      });
      if (literalObjectProperty) {
        literalProperty = literalObjectProperty.value;
      }
    } else if (t.isArrayExpression(literalObject)) {
      literalProperty = literalObject.elements[pathPropertyValue];
    } else {
      throw new Error(
      // eslint-disable-next-line max-len
      `${literalObject.type} is not yet supported in memberExpressionToLiteral`);
    }
    return t.clone(literalProperty || t.identifier('undefined'));

    function getPathPropertyValue() {
      if (pathProperty.isIdentifier() && path.node.computed) {
        return getLiteralPropertyValue(identifierToLiteral(pathProperty));
      } else {
        return getLiteralPropertyValue(pathProperty.node);
      }
    }

    function getLiteralPropertyValue(node) {
      if (t.isLiteral(node)) {
        return node.value;
      } else if (t.isIdentifier(node)) {
        return node.name;
      } else {
        throw new Error(
        // eslint-disable-next-line max-len
        `${node.type} is not yet supported in getLiteralPropertyValue of memberExpressionToLiteral`);
      }
    }
  }

  function toLiteral(path) {
    var toLiterals = [[t.isLiteral, function (p) {
      return t.clone(p.node);
    }], [t.isMemberExpression, memberExpressionToLiteral], [t.isIdentifier, identifierToLiteral], [t.isArrayExpression, arrayToLiteral], [t.isObjectExpression, objectToLiteral]];
    return toLiterals.reduce(function (literalVal, _ref2) {
      var _ref3 = _slicedToArray(_ref2, 2),
          test = _ref3[0],
          toLiteralFn = _ref3[1];

      if (literalVal) {
        return literalVal;
      }
      if (test(path)) {
        return toLiteralFn(path);
      }
      return null;
    }, null);
  }
}