'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (babel) {
  var t = babel.types;

  var glamorousIdentifiers = new Set();
  return {
    name: 'glamorous-static',
    visitor: {
      ImportDeclaration(path) {
        var defaultSpecifierPath = path.get('specifiers')[0];
        if (path.node.source.value !== 'glamorous' || !t.isImportDefaultSpecifier(defaultSpecifierPath)) {
          return;
        }
        var name = defaultSpecifierPath.node.local.name;

        var _path$scope$getBindin = path.scope.getBinding(name),
            referencePaths = _path$scope$getBindin.referencePaths;

        referencePaths.forEach(function (reference) {
          glamorousIdentifiers.add(reference);
        });
      },
      VariableDeclarator(path) {
        var node = path.node;

        if (!isRequireCall(node.init) || !t.isIdentifier(node.id)) {
          return;
        }
        var name = node.id.name;

        var binding = path.scope.getBinding(name);
        var referencePaths = binding.referencePaths;

        referencePaths.forEach(function (reference) {
          glamorousIdentifiers.add(reference);
        });
      },
      Program: {
        exit(programPath, context) {
          var toLiteral = (0, _getLiteralizers2.default)(context);
          Array.from(glamorousIdentifiers).forEach(function (identifier) {
            var isGlamorousCall = looksLike(identifier, {
              parentPath: {
                type: function type(_type) {
                  return _type === 'MemberExpression' || _type === 'CallExpression';
                },
                parentPath: {
                  type: 'CallExpression'
                }
              }
            });
            if (!isGlamorousCall) {
              return;
            }
            var callExpression = identifier.parentPath.parentPath;
            var staticPaths = callExpression.get('arguments').reduce(function (paths, argPath) {
              return paths.concat(getStaticPaths(argPath));
            }, []).filter(Boolean);
            staticPaths.forEach(function (staticPath) {
              staticPath.replaceWith(t.stringLiteral(glamorize(staticPath)));
            });

            function glamorize(literalNodePath) {
              var obj = void 0;

              var _babel$transformFromA = babel.transformFromAst(t.program([t.expressionStatement(toLiteral(literalNodePath))])),
                  code = _babel$transformFromA.code;
              // eslint-disable-next-line no-eval


              eval(`obj = ${code}`);
              var className = glamor.css(obj).toString();
              return className;
            }
          });

          // babel utils
          function getStaticPaths(path) {
            var pathGetters = [getStaticObjectPaths, getStaticsInDynamic, getStaticReferences];
            return pathGetters.reduce(function (pathsAlreadyGotten, pathGetter) {
              if (pathsAlreadyGotten) {
                return pathsAlreadyGotten;
              }
              var paths = pathGetter(path);
              if (paths.length) {
                return paths;
              }
              return null;
            }, null);
          }

          function getStaticObjectPaths(path) {
            if (path.type !== 'ObjectExpression') {
              return [];
            }
            return [path];
          }

          function getStaticsInDynamic(path) {
            var isSupportedFunction = looksLike(path, {
              node: {
                type: 'ArrowFunctionExpression',
                body: {
                  type: 'ConditionalExpression',
                  consequent: { type: 'ObjectExpression' },
                  alternate: { type: 'ObjectExpression' }
                }
              }
            });
            if (!isSupportedFunction) {
              return [];
            }
            var things = [].concat(_toConsumableArray(getStaticPaths(path.get('body.consequent'))), _toConsumableArray(getStaticPaths(path.get('body.alternate'))));
            return things;
          }

          function getStaticReferences(path) {
            if (path.type !== 'Identifier') {
              return [];
            }
            var binding = path.scope.getBinding(path.node.name);
            if (binding) {
              return getStaticObjectPaths(binding.path.get('init'));
            } else {
              return [];
            }
          }
        }
      }
    }
  };
};

var _glamor = require('glamor');

var glamor = _interopRequireWildcard(_glamor);

var _getLiteralizers = require('./get-literalizers');

var _getLiteralizers2 = _interopRequireDefault(_getLiteralizers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function isRequireCall(callExpression) {
  return looksLike(callExpression, {
    type: 'CallExpression',
    callee: {
      name: 'require'
    },
    arguments: function _arguments(args) {
      return args.length === 1 && looksLike(args[0], { value: 'glamorous' });
    }
  });
}

// generic utils
function looksLike(a, b) {
  return a && b && Object.keys(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === 'function') {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  // eslint-disable-next-line
  return val == null || /^[sbn]/.test(typeof val);
}